@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,6 +22,12 @@
 */
 
 #include "swarm.h"
+#include "db.h"
+#include "qgram.h"
+#include "matrix.h"
+#include "nw.h"
+#include "scan.h"
+#include "util.h"
 
 static uint64_t count_comparisons_8;
 static uint64_t count_comparisons_16;
@@ -78,8 +84,117 @@ auto compare_mass_seed(const void * a, const void * b) -> int
   return status;
 }
 
+
+auto write_representative_sequences(const uint64_t amplicons,
+                                    struct Parameters const & p) -> void {
+
+  uint64_t swarmcount {0};
+  progress_init("Sorting seeds:    ", amplicons);
+  auto * swarminfo = new struct swarminfo_t[swarmed];
+  uint64_t mass {0};
+  unsigned previd = amps[0].swarmid;
+  unsigned seed = amps[0].ampliconid;
+  mass += db_getabundance(seed);
+  for(auto i = 1ULL; i < amplicons; i++)
+    {
+      unsigned id = amps[i].swarmid;
+      if (id != previd)
+        {
+          swarminfo[swarmcount].seed = seed;
+          swarminfo[swarmcount].mass = mass;
+          swarmcount++;
+          mass = 0;
+          seed = amps[i].ampliconid;
+        }
+      mass += db_getabundance(amps[i].ampliconid);
+      previd = id;
+      progress_update(i);
+    }
+  swarminfo[swarmcount].seed = seed;
+  swarminfo[swarmcount].mass = mass;
+  swarmcount++;
+  qsort(swarminfo, swarmcount, sizeof(swarminfo_t), compare_mass_seed);
+  progress_done();
+
+  progress_init("Writing seeds:    ", swarmcount);
+  for(auto i = 0ULL; i < swarmcount; i++)
+    {
+      uint64_t swarm_mass = swarminfo[i].mass;
+      unsigned int swarm_seed = swarminfo[i].seed;
+
+      fprintf(fp_seeds, ">");
+      fprint_id_with_new_abundance(fp_seeds, swarm_seed, swarm_mass, p.opt_usearch_abundance);
+      fprintf(fp_seeds, "\n");
+      db_fprintseq(fp_seeds, swarm_seed);
+      progress_update(i);
+    }
+  delete [] swarminfo;
+  swarminfo = nullptr;
+  progress_done();
+}
+
+
+auto write_swarms_default_format(const uint64_t amplicons,
+                                 struct Parameters const & p) -> void {
+  /* native swarm output */
+  constexpr char sep_amplicons {sepchar};  /* usually a space */
+  constexpr char sep_swarms {'\n'};
+
+  fprint_id(outfile, amps[0].ampliconid,
+            p.opt_usearch_abundance, p.opt_append_abundance);
+  int64_t previd = amps[0].swarmid;
+
+  for(auto i = 1ULL; i < amplicons; i++)
+    {
+      int64_t id = amps[i].swarmid;
+      if (id == previd) {
+        fputc(sep_amplicons, outfile);
+      }
+      else {
+        fputc(sep_swarms, outfile);
+      }
+      fprint_id(outfile, amps[i].ampliconid,
+                p.opt_usearch_abundance, p.opt_append_abundance);
+      previd = id;
+    }
+  fputc('\n', outfile);
+}
+
+
+auto write_swarms_mothur_format(const uint64_t amplicons,
+                                const unsigned int swarmid,
+                                struct Parameters const & p) -> void {
+  /* mothur list file output */
+  constexpr char sep_amplicons {','};
+  constexpr char sep_swarms {'\t'};
+
+  fprintf(outfile, "swarm_%" PRId64 "\t%u\t", p.opt_differences, swarmid);
+
+  fprint_id(outfile, amps[0].ampliconid,
+            p.opt_usearch_abundance, p.opt_append_abundance);
+  int64_t previd = amps[0].swarmid;
+
+  for(auto i = 1ULL; i < amplicons; i++)
+    {
+      int64_t id = amps[i].swarmid;
+      if (id == previd) {
+        fputc(sep_amplicons, outfile);
+      }
+      else {
+        fputc(sep_swarms, outfile);
+      }
+      fprint_id(outfile, amps[i].ampliconid,
+                p.opt_usearch_abundance, p.opt_append_abundance);
+      previd = id;
+    }
+
+  fputc('\n', outfile);
+}
+
+
 void algo_run(struct Parameters const & p)
 {
+  score_matrix_read(p);
   search_begin();
 
   count_comparisons_8 = 0;
@@ -101,32 +216,19 @@ void algo_run(struct Parameters const & p)
 
   qgram_diff_init();
 
-  amps = static_cast<struct ampliconinfo_s *>
-    (xmalloc(amplicons * sizeof(struct ampliconinfo_s)));
-
-  targetampliconids = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-  targetindices = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-  scores = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-  diffs = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-  alignlengths = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-
-  qgramamps = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-  qgramdiffs = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-  qgramindices = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
-
-  auto * hits = static_cast<uint64_t *>
-    (xmalloc(amplicons * sizeof(uint64_t)));
+  amps = new struct ampliconinfo_s[amplicons];
+  targetampliconids = new uint64_t[amplicons];
+  targetindices = new uint64_t[amplicons];
+  scores = new uint64_t[amplicons];
+  diffs = new uint64_t[amplicons];
+  alignlengths = new uint64_t[amplicons];
+  qgramamps = new uint64_t[amplicons];
+  qgramdiffs = new uint64_t[amplicons];
+  qgramindices = new uint64_t[amplicons];
+  auto * hits = new uint64_t[amplicons];
 
   auto diff_saturation
-    = static_cast<uint64_t>(std::min(UINT8_MAX / penalty_mismatch,
+    = static_cast<uint64_t>(std::min(UINT8_MAX / p.penalty_mismatch,
                                      UINT8_MAX / (penalty_gapopen +
                                                   penalty_gapextend)));
 
@@ -135,10 +237,8 @@ void algo_run(struct Parameters const & p)
 
   if (uclustfile != nullptr)
     {
-      dir = static_cast<unsigned char *>
-        (xmalloc(longestamplicon*longestamplicon));
-      hearray = static_cast<uint64_t *>
-        (xmalloc(2 * longestamplicon * sizeof(uint64_t)));
+      dir = new unsigned char[longestamplicon * longestamplicon];
+      hearray = new uint64_t[2 * longestamplicon];
     }
 
   /* set ampliconid for all */
@@ -303,10 +403,10 @@ void algo_run(struct Parameters const & p)
                   if (! p.opt_internal_structure.empty())
                     {
                       fprint_id_noabundance(internal_structure_file,
-                                            seedampliconid);
+                                            seedampliconid, p.opt_usearch_abundance);
                       fprintf(internal_structure_file, "\t");
                       fprint_id_noabundance(internal_structure_file,
-                                            poolampliconid);
+                                            poolampliconid, p.opt_usearch_abundance);
                       fprintf(internal_structure_file, "\t%" PRIu64, diff);
                       fprintf(internal_structure_file,
                               "\t%u\t1",
@@ -449,10 +549,12 @@ void algo_run(struct Parameters const & p)
                           if (! p.opt_internal_structure.empty())
                             {
                               fprint_id_noabundance(internal_structure_file,
-                                                    subseedampliconid);
+                                                    subseedampliconid,
+                                                    p.opt_usearch_abundance);
                               fprintf(internal_structure_file, "\t");
                               fprint_id_noabundance(internal_structure_file,
-                                                    poolampliconid);
+                                                    poolampliconid,
+                                                    p.opt_usearch_abundance);
                               fprintf(internal_structure_file, "\t%" PRIu64, diff);
                               fprintf(internal_structure_file,
                                       "\t%u\t%" PRIu64,
@@ -488,12 +590,12 @@ void algo_run(struct Parameters const & p)
         {
           fprintf(uclustfile, "C\t%u\t%" PRIu64 "\t*\t*\t*\t*\t*\t",
                   swarmid-1, swarmsize);
-          fprint_id(uclustfile, seedampliconid);
+          fprint_id(uclustfile, seedampliconid, p.opt_usearch_abundance, p.opt_append_abundance);
           fprintf(uclustfile, "\t*\n");
 
           fprintf(uclustfile, "S\t%u\t%u\t*\t*\t*\t*\t*\t",
                   swarmid-1, db_getsequencelen(seedampliconid));
-          fprint_id(uclustfile, seedampliconid);
+          fprint_id(uclustfile, seedampliconid, p.opt_usearch_abundance, p.opt_append_abundance);
           fprintf(uclustfile, "\t*\n");
           fflush(uclustfile);
 
@@ -524,15 +626,14 @@ void algo_run(struct Parameters const & p)
                       swarmid-1, db_getsequencelen(hit), percentid,
                       nwdiff > 0 ? nwalignment : "=");
 
-              fprint_id(uclustfile, hit);
+              fprint_id(uclustfile, hit, p.opt_usearch_abundance, p.opt_append_abundance);
               fprintf(uclustfile, "\t");
-              fprint_id(uclustfile, seedampliconid);
+              fprint_id(uclustfile, seedampliconid, p.opt_usearch_abundance, p.opt_append_abundance);
               fprintf(uclustfile, "\n");
               fflush(uclustfile);
 
-              if (nwalignment != nullptr) {
-                xfree(nwalignment);
-              }
+              delete [] nwalignment;
+              nwalignment = nullptr;
             }
 
         }
@@ -544,7 +645,7 @@ void algo_run(struct Parameters const & p)
 
           fprintf(statsfile, "%" PRIu64 "\t%" PRIu64 "\t",
                   swarmsize, amplicons_copies);
-          fprint_id_noabundance(statsfile, seedampliconid);
+          fprint_id_noabundance(statsfile, seedampliconid, p.opt_usearch_abundance);
           fprintf(statsfile,
                   "\t%" PRIu64 "\t%" PRIu64 "\t%" PRIu64 "\t%" PRIu64 "\n",
                   abundance, singletons, maxgen, maxradius);
@@ -555,113 +656,50 @@ void algo_run(struct Parameters const & p)
 
   if (uclustfile != nullptr)
     {
-      xfree(dir);
-      xfree(hearray);
+      delete [] dir;
+      dir = nullptr;
+      delete [] hearray;
+      hearray = nullptr;
     }
 
 
-  /* output results */
-
-  if (amplicons > 0)
-    {
-      char sep_amplicons {0};
-      char sep_swarms {0};
-
-      if (opt_mothur)
-        {
-          /* mothur list file output */
-          sep_amplicons = ',';
-          sep_swarms = '\t';
-          fprintf(outfile, "swarm_%" PRId64 "\t%u\t",
-                  p.opt_differences, swarmid);
-        }
-      else
-        {
-          /* native swarm output */
-          sep_amplicons = sepchar;  /* usually a space */
-          sep_swarms = '\n';
-        }
-
-      fprint_id(outfile, amps[0].ampliconid);
-      int64_t previd = amps[0].swarmid;
-
-      for(auto i = 1ULL; i < amplicons; i++)
-        {
-          int64_t id = amps[i].swarmid;
-          if (id == previd) {
-            fputc(sep_amplicons, outfile);
-          }
-          else {
-            fputc(sep_swarms, outfile);
-          }
-          fprint_id(outfile, amps[i].ampliconid);
-          previd = id;
-        }
-
-      fputc('\n', outfile);
+  /* output swarms */
+  if (amplicons > 0) {
+    if (p.opt_mothur) {
+      write_swarms_mothur_format(amplicons, swarmid, p);
     }
+    else {
+      write_swarms_default_format(amplicons, p);
+    }
+  }
 
 
   /* dump seeds in fasta format with sum of abundances */
-
-  if ((! opt_seeds.empty()) && (amplicons > 0))
-    {
-      uint64_t swarmcount {0};
-      progress_init("Sorting seeds:    ", amplicons);
-      auto * swarminfo = static_cast<struct swarminfo_t *>
-        (xmalloc(swarmed * sizeof(struct swarminfo_t)));
-      uint64_t mass {0};
-      unsigned previd = amps[0].swarmid;
-      unsigned seed = amps[0].ampliconid;
-      mass += db_getabundance(seed);
-      for(auto i = 1ULL; i < amplicons; i++)
-        {
-          unsigned id = amps[i].swarmid;
-          if (id != previd)
-            {
-              swarminfo[swarmcount].seed = seed;
-              swarminfo[swarmcount].mass = mass;
-              swarmcount++;
-              mass = 0;
-              seed = amps[i].ampliconid;
-            }
-          mass += db_getabundance(amps[i].ampliconid);
-          previd = id;
-          progress_update(i);
-        }
-      swarminfo[swarmcount].seed = seed;
-      swarminfo[swarmcount].mass = mass;
-      swarmcount++;
-      qsort(swarminfo, swarmcount, sizeof(swarminfo_t), compare_mass_seed);
-      progress_done();
-
-      progress_init("Writing seeds:    ", swarmcount);
-      for(auto i = 0ULL; i < swarmcount; i++)
-        {
-          uint64_t swarm_mass = swarminfo[i].mass;
-          unsigned int swarm_seed = swarminfo[i].seed;
-
-          fprintf(fp_seeds, ">");
-          fprint_id_with_new_abundance(fp_seeds, swarm_seed, swarm_mass);
-          fprintf(fp_seeds, "\n");
-          db_fprintseq(fp_seeds, swarm_seed, 0);
-          progress_update(i);
-        }
-      xfree(swarminfo);
-      progress_done();
-    }
+  if ((not p.opt_seeds.empty()) && (amplicons > 0)) {
+    write_representative_sequences(amplicons, p);
+  }
 
 
-  xfree(qgramdiffs);
-  xfree(qgramamps);
-  xfree(qgramindices);
-  xfree(hits);
-  xfree(alignlengths);
-  xfree(diffs);
-  xfree(scores);
-  xfree(targetindices);
-  xfree(targetampliconids);
-  xfree(amps);
+  delete [] qgramdiffs;
+  qgramdiffs = nullptr;
+  delete [] qgramamps;
+  qgramamps = nullptr;
+  delete [] qgramindices;
+  qgramindices = nullptr;
+  delete [] hits;
+  hits = nullptr;
+  delete [] alignlengths;
+  alignlengths = nullptr;
+  delete [] diffs;
+  diffs = nullptr;
+  delete [] scores;
+  scores = nullptr;
+  delete [] targetindices;
+  targetindices = nullptr;
+  delete [] targetampliconids;
+  targetampliconids = nullptr;
+  delete [] amps;
+  amps = nullptr;
 
   db_qgrams_done();
 
@@ -699,5 +737,6 @@ void algo_run(struct Parameters const & p)
 #endif
 
     search_end();
+    score_matrix_free();
 }
 
