@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -31,27 +31,25 @@
   https://doi.org/10.1145/1498698.1594230
 */
 
-#include "swarm.h"
+#include <cstdint>
+#include <cstring>
+#include "bloomflex.h"
+#include "pseudo_rng.h"
+#include "util.h"
 
-void bloomflex_patterns_generate(struct bloomflex_s * b);
 
-void bloomflex_patterns_generate(struct bloomflex_s * b)
+auto bloomflex_patterns_generate(struct bloomflex_s * b) -> void
 {
   constexpr unsigned int max_range {63};  // i & max_range = cap values to 63 max
-#if 0
-  printf("Generating %" PRIu64 " patterns with %" PRIu64 " bits set.\n",
-         b->pattern_count,
-         b->pattern_k);
-#endif
   for(auto i = 0U; i < b->pattern_count; i++)
     {
       uint64_t pattern {0};
       for(auto j = 0U; j < b->pattern_k; j++)
         {
           uint64_t onebit {0};
-          onebit = 1ULL << (arch_random() & max_range);  // 0 <= shift <= 63
+          onebit = 1ULL << (rand_64() & max_range);  // 0 <= shift <= 63
           while ((pattern & onebit) != 0U) {
-            onebit = 1ULL << (arch_random() & max_range);
+            onebit = 1ULL << (rand_64() & max_range);
           }
           pattern |= onebit;
         }
@@ -59,12 +57,14 @@ void bloomflex_patterns_generate(struct bloomflex_s * b)
     }
 }
 
+
 auto bloomflex_init(uint64_t size, unsigned int k) -> struct bloomflex_s *
 {
   /* Input size is in bytes for full bitmap */
 
   constexpr unsigned int multiplier {16};  // multiply by 65,536
   constexpr unsigned int divider {3};  // divide by 8
+
   auto * b = static_cast<struct bloomflex_s *>(xmalloc(sizeof(struct bloomflex_s)));
   b->size = size >> divider;
 
@@ -82,7 +82,8 @@ auto bloomflex_init(uint64_t size, unsigned int k) -> struct bloomflex_s *
   return b;
 }
 
-void bloomflex_exit(struct bloomflex_s * b)
+
+auto bloomflex_exit(struct bloomflex_s * b) -> void
 {
   xfree(b->bitmap);
   xfree(b->patterns);
