@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -31,23 +31,25 @@
   https://doi.org/10.1145/1498698.1594230
 */
 
-#include "swarm.h"
+#include <cstdint>
+#include <cstring>
+#include "bloompat.h"
+#include "pseudo_rng.h"
 
-void bloom_patterns_generate(struct bloom_s * b);
 
 void bloom_patterns_generate(struct bloom_s * b)
 {
   constexpr unsigned int max_range {63};  // i & max_range = cap values to 63 max
-  const unsigned int k {8};
-  for(auto i = 0U; i < bloom_pattern_count; i++)
+  constexpr unsigned int k {8};
+  for(auto i = 0U; i < bloom_pattern_count; i++)  // range-loop '(unsigned long & i : b->patterns)' -> FAIL issue 123??
     {
       uint64_t pattern {0};
       for(auto j = 0U; j < k; j++)
         {
           uint64_t onebit {0};
-          onebit = 1ULL << (arch_random() & max_range);  // 0 <= shift <= 63
+          onebit = 1ULL << (rand_64() & max_range);  // 0 <= shift <= 63
           while ((pattern & onebit) != 0) {
-            onebit = 1ULL << (arch_random() & max_range);
+            onebit = 1ULL << (rand_64() & max_range);
           }
           pattern |= onebit;
         }
@@ -55,24 +57,27 @@ void bloom_patterns_generate(struct bloom_s * b)
     }
 }
 
+
 void bloom_zap(struct bloom_s * b)
 {
   memset(b->bitmap, UINT8_MAX, b->size);
 }
 
+
 auto bloom_init(uint64_t size) -> struct bloom_s *
 {
-  // Size is in bytes for full bitmap, must be power of 2
+  // Size is in bytes for full bitmap, must be power of 2,
   // at least 8
-  size = std::max(size, static_cast<uint64_t>(8));
+  constexpr uint64_t bytes_per_uint64 {8};
+  size = std::max(size, bytes_per_uint64);
 
-  auto * b = static_cast<struct bloom_s *>(xmalloc(sizeof(struct bloom_s)));
+  auto * b = new struct bloom_s;
 
   b->size = size;
 
   b->mask = (size >> 3) - 1;
 
-  b->bitmap = static_cast<uint64_t *>(xmalloc(size));
+  b->bitmap = new uint64_t[size];
 
   bloom_zap(b);
 
@@ -81,8 +86,11 @@ auto bloom_init(uint64_t size) -> struct bloom_s *
   return b;
 }
 
+
 void bloom_exit(struct bloom_s * b)
 {
-  xfree(b->bitmap);
-  xfree(b);
+  delete [] b->bitmap;
+  b->bitmap = nullptr;
+  delete b;
+  b = nullptr;
 }
