@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,14 +22,24 @@
 */
 
 #include "swarm.h"
+#include "qgram.h"
+#include "util.h"
+#include "utils/fatal.h"
+#include "zobrist.h"
+
 
 constexpr unsigned int memchunk {1 << 20};  // 1 megabyte
 constexpr unsigned int linealloc {2048};
 constexpr long unsigned int n_chars {INT8_MAX + 1};  // 128 ascii chars
+constexpr unsigned int max_sequence_length {67108861};  // (2^26 - 3)
+// for longer sequences, 'zobrist_tab_byte_base' is bigger than 8 x
+// 2^32 (512 x max_sequence_length) and cannot be addressed with
+// uint32 pointers, which leads to a segmentation fault
+constexpr unsigned int max_header_length {16777216 - 1};  // 2^24 minus 1
 
 auto make_nt_map () -> std::array<signed char, n_chars> {
     // set the 128 ascii chars to '-1' except Aa, Cc, Gg, Tt and Uu
-    std::array<signed char, n_chars> ascii_map;
+  std::array<signed char, n_chars> ascii_map {{0}};
     ascii_map.fill(-1);
     ascii_map['A'] = 1;
     ascii_map['a'] = 1;
@@ -46,6 +56,13 @@ auto make_nt_map () -> std::array<signed char, n_chars> {
 
 const auto map_nt = make_nt_map();
 
+const std::array<char, 32> sym_nt =
+  {'-', 'A', 'C', 'G', 'T', ' ', ' ', ' ',
+   ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
+   ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
+   ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '};
+
+
 static unsigned int sequences {0};
 static uint64_t nucleotides {0};
 static uint64_t headerchars {0};
@@ -56,29 +73,54 @@ static int missingabundance {0};
 static uint64_t missingabundance_lineno {0};
 static char * missingabundance_header {nullptr};
 
+struct seqinfo_s
+{
+  char * header;
+  char * seq;
+  uint64_t abundance;
+  uint64_t hdrhash;
+  uint64_t seqhash;
+  int headerlen;
+  unsigned int seqlen;
+  unsigned int clusterid;
+  int abundance_start;
+  int abundance_end;
+  int dummy; /* alignment padding only */
+};
+
+using seqinfo_t = struct seqinfo_s;
+extern seqinfo_t * seqindex;
+
 seqinfo_t * seqindex {nullptr};
 qgramvector_t * qgrams {nullptr};
 
-auto db_compare_abundance(const void * a, const void * b) -> int;
 
-auto find_swarm_abundance(const char * header,
-                          int * start,
-                          int * end,
-                          int64_t * number) -> bool;
-
-auto find_usearch_abundance(const char * header,
-                            int * start,
-                            int * end,
-                            int64_t * number) -> bool;
-void find_abundance(struct seqinfo_s * sp, uint64_t lineno);
-
-void fprint_id(FILE * stream, uint64_t x)
+auto db_getnucleotidecount() -> uint64_t
 {
-  seqinfo_t * sp = seqindex + x;
-  char * h = sp->header;
-  int hdrlen = sp->headerlen;
+  return nucleotides;
+}
 
-  if (opt_append_abundance != 0 && (sp->abundance_start == sp->abundance_end)) {
+
+auto db_getsequencecount() -> unsigned int
+{
+  return sequences;
+}
+
+
+auto db_getlongestsequence() -> unsigned int
+{
+  return longest;
+}
+
+
+auto fprint_id(std::FILE * stream, uint64_t x, bool opt_usearch_abundance,
+               int64_t opt_append_abundance) -> void
+{
+  const seqinfo_t * sp = seqindex + x;
+  const char * h = sp->header;
+  const int hdrlen = sp->headerlen;
+
+  if ((opt_append_abundance != 0) && (sp->abundance_start == sp->abundance_end)) {
     if (opt_usearch_abundance) {
       fprintf(stream, "%.*s;size=%" PRIu64 ";", hdrlen, h, sp->abundance);
     }
@@ -91,11 +133,12 @@ void fprint_id(FILE * stream, uint64_t x)
   }
 }
 
-void fprint_id_noabundance(FILE * stream, uint64_t x)
+
+auto fprint_id_noabundance(std::FILE * stream, uint64_t seqno, bool opt_usearch_abundance) -> void
 {
-  seqinfo_t * sp = seqindex + x;
-  char * h = sp->header;
-  int hdrlen = sp->headerlen;
+  const seqinfo_t * sp = seqindex + seqno;
+  const char * h = sp->header;
+  const int hdrlen = sp->headerlen;
 
   if (sp->abundance_start < sp->abundance_end)
     {
@@ -118,11 +161,13 @@ void fprint_id_noabundance(FILE * stream, uint64_t x)
   }
 }
 
-void fprint_id_with_new_abundance(FILE * stream,
+
+auto fprint_id_with_new_abundance(std::FILE * stream,
                                   uint64_t seqno,
-                                  uint64_t abundance)
+                                  uint64_t abundance,
+                                  bool opt_usearch_abundance) -> void
 {
-  seqinfo_t * sp = seqindex + seqno;
+  const seqinfo_t * sp = seqindex + seqno;
 
   if (opt_usearch_abundance) {
     fprintf(stream,
@@ -143,6 +188,7 @@ void fprint_id_with_new_abundance(FILE * stream,
   }
 }
 
+
 auto db_compare_abundance(const void * a, const void * b) -> int
 {
   const auto * x = reinterpret_cast<const seqinfo_t *>(a);
@@ -161,6 +207,7 @@ auto db_compare_abundance(const void * a, const void * b) -> int
   return status;
 }
 
+
 auto find_swarm_abundance(const char * header,
                           int * start,
                           int * end,
@@ -208,6 +255,7 @@ auto find_swarm_abundance(const char * header,
   return true;
 }
 
+
 auto find_usearch_abundance(const char * header,
                             int * start,
                             int * end,
@@ -218,15 +266,12 @@ auto find_usearch_abundance(const char * header,
     in the header string.
   */
 
-  if (header == nullptr) {
-    return false;
-  }
+  assert(header != nullptr); // header cannot be a nullptr at this stage
 
   const std::string attribute {"size="};
-  const std::string digit_chars = "0123456789";
-
-  uint64_t hlen = strlen(header);
-  uint64_t alen = attribute.length();
+  const std::string digit_chars {"0123456789"};
+  const uint64_t hlen = strlen(header);
+  const uint64_t alen = attribute.length();
   uint64_t i = 0;
 
   while (i + alen < hlen)
@@ -278,7 +323,9 @@ auto find_usearch_abundance(const char * header,
   return false;
 }
 
-void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
+
+void find_abundance(struct seqinfo_s * sp, uint64_t lineno,
+                    bool opt_usearch_abundance, int64_t opt_append_abundance)
 {
   char * header = sp->header;
 
@@ -294,18 +341,11 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
 
       if (find_usearch_abundance(header, & start, & end, & number))
         {
-          if (number > 0) {
-            abundance = number;
+          if (number <= 0) {
+            fatal(error_prefix, "Illegal abundance value on line ", lineno, ":\n",
+                  header, "\nAbundance values should be positive integers.");
           }
-          else
-            {
-              fprintf(stderr,
-                      "\nError: Illegal abundance value on line %" PRIu64 ":\n%s\n"
-                      "Abundance values should be positive integers.\n\n",
-                      lineno,
-                      header);
-              exit(1);
-            }
+          abundance = number;
         }
     }
   else
@@ -314,18 +354,11 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
 
       if (find_swarm_abundance(header, & start, & end, & number))
         {
-          if (number > 0) {
-            abundance = number;
+          if (number <= 0) {
+            fatal(error_prefix, "Illegal abundance value on line ", lineno, ":\n",
+                  header, "\nAbundance values should be positive integers.");
           }
-          else
-            {
-              fprintf(stderr,
-                      "\nError: Illegal abundance value on line %" PRIu64 ":\n%s\n"
-                      "Abundance values should be positive integers.\n\n",
-                      lineno,
-                      header);
-              exit(1);
-            }
+          abundance = number;
         }
     }
 
@@ -340,6 +373,7 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
       else
         {
           missingabundance++;
+          // record the position of the first missing abundance entry
           if (missingabundance == 1)
             {
               missingabundance_lineno = lineno;
@@ -353,13 +387,15 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
   sp->abundance_end = end;
 }
 
+
 void db_read(const char * filename, struct Parameters const & p)
 {
   /* allocate space */
 
   uint64_t dataalloc {memchunk};
-  datap = static_cast<char *>(xmalloc(dataalloc));
+  datap = new char[dataalloc];
   uint64_t datalen {0};
+  uint64_t duplicates_found {0};
 
   longest = 0;
   longestheader = 0;
@@ -367,45 +403,45 @@ void db_read(const char * filename, struct Parameters const & p)
   nucleotides = 0;
   headerchars = 0;
 
-  FILE * fp {nullptr};
-  if (filename != nullptr)
+  /* open input file or stream */
+
+  assert(filename != nullptr);  // filename is set to '-' (stdin) by default
+
+  std::FILE * input_fp { fopen_input(filename) };
+  if (input_fp == nullptr)
     {
-      fp = fopen_input(filename);
-      if (fp == nullptr)
-        {
-          fprintf(stderr, "\nError: Unable to open input data file (%s).\n", filename);
-          exit(1);
-        }
+      fatal(error_prefix, "Unable to open input data file (", filename, ").\n");
     }
-  else {
-    fp = stdin;
-  }
 
   /* get file size */
 
   struct stat fs;
 
-  if (fstat(fileno(fp), & fs) != 0)
+  if (fstat(fileno(input_fp), & fs) != 0)
     {
-      fprintf(stderr, "\nUnable to fstat on input file (%s)\n", filename);
-      exit(1);
+      fatal(error_prefix, "Unable to fstat on input file (", filename, ").\n");
     }
   bool is_regular = S_ISREG(fs.st_mode);
-  int64_t filesize = is_regular ? fs.st_size : 0;
+  uint64_t filesize = is_regular ? fs.st_size : 0;
+  uint64_t filepos = 0;
 
   if (! is_regular) {
-    fprintf(logfile, "Waiting for data... (Hit Ctrl-C and run swarm -h if you meant to read data from a file.)\n");
+    fprintf(logfile, "Waiting for data... (hit Ctrl-C and run 'swarm -h' if you meant to read data from a file)\n");
   }
 
-  char line[linealloc];
-  line[0] = 0;
-  if (fgets(line, linealloc, fp) == nullptr) {
-    line[0] = 0;
-  }
+  size_t linecap = linealloc;
+  char * line {new char[linecap]};
+  ssize_t linelen = xgetline(& line, & linecap, input_fp);
+  if (linelen < 0)
+    {
+      line[0] = 0;
+      linelen = 0;
+    }
+  filepos += linelen;
 
   unsigned int lineno {1};
 
-  progress_init("Reading sequences:", static_cast<uint64_t>(filesize));
+  progress_init("Reading sequences:", filesize);
 
   while(line[0] != 0)
     {
@@ -413,22 +449,21 @@ void db_read(const char * filename, struct Parameters const & p)
       /* the header ends at a space, cr, lf or null character */
 
       if (line[0] != '>') {
-        fatal("Illegal header line in fasta file.");
+        fatal(error_prefix, "Illegal header line in fasta file.");
       }
 
       auto headerlen = static_cast<unsigned int>
         (strcspn(line + 1, " \r\n"));
 
-      if (headerlen >= linealloc - 2) {
-        fatal("The FASTA header line is too long.");
-      }
-
       headerchars += headerlen;
 
       if (headerlen > longestheader) {
         longestheader = headerlen;
       }
 
+      if (longestheader > max_header_length) {
+        fatal(error_prefix, "Headers longer than 16,777,215 symbols are not supported.");
+      }
 
       /* store the line number */
 
@@ -455,10 +490,14 @@ void db_read(const char * filename, struct Parameters const & p)
 
       /* get next line */
 
-      line[0] = 0;
-      if (fgets(line, linealloc, fp) == nullptr) {
-        line[0] = 0;
-      }
+      linelen = xgetline(& line, & linecap, input_fp);
+      if (linelen < 0)
+        {
+          line[0] = 0;
+          linelen = 0;
+        }
+      filepos += linelen;
+
       lineno++;
 
 
@@ -517,25 +556,29 @@ void db_read(const char * filename, struct Parameters const & p)
               else if ((c != new_line) && (c != carriage_return))
                 {
                   if ((c >= start_chars_range) && (c <= end_chars_range)) {
-                    fprintf(stderr,
-                            "\nError: Illegal character '%c' in sequence on line %u\n",
-                            c,
-                            lineno);
+                    fatal(error_prefix, "Illegal character '", static_cast<char>(c),
+                          "' in sequence on line ", lineno, ".");
                   }
                   else {
-                    fprintf(stderr,
-                            "\nError: Illegal character (ascii no %d) in sequence on line %u\n",
-                            c,
-                            lineno);
+                    fatal(error_prefix, "Illegal character (ascii no ", static_cast<char>(c),
+                          ") in sequence on line ", lineno, ".");
                   }
-                  exit(1);
                 }
             }
 
-          line[0] = 0;
-          if (fgets(line, linealloc, fp) == nullptr) {
-            line[0] = 0;
+          /* check length of longest sequence */
+          if (length > max_sequence_length) {
+            fatal(error_prefix, "Sequences longer than 67,108,861 symbols are not supported.");
           }
+
+          linelen = xgetline(& line, & linecap, input_fp);
+          if (linelen < 0)
+            {
+              line[0] = 0;
+              linelen = 0;
+            }
+          filepos += linelen;
+
           lineno++;
         }
 
@@ -545,8 +588,7 @@ void db_read(const char * filename, struct Parameters const & p)
 
       if (length == 0)
         {
-          fprintf(stderr, "\nError: Empty sequence found on line %u.\n\n", lineno-1);
-          exit(1);
+          fatal(error_prefix, "Empty sequence found on line ", lineno - 1, ".");
         }
 
       nucleotides += length;
@@ -570,18 +612,18 @@ void db_read(const char * filename, struct Parameters const & p)
           datalen += sizeof(nt_buffer);
 
           nt_buffer = 0;
-          nt_bufferlen = 0;
+          nt_bufferlen = 0;  // that value is never read again, all tests pass without it
         }
 
       sequences++;
 
       if (is_regular) {
-        progress_update(static_cast<uint64_t>(ftell(fp)));
+        progress_update(filepos);
       }
     }
   progress_done();
 
-  fclose(fp);
+  fclose(input_fp);
 
   /* init zobrist hashing */
 
@@ -593,11 +635,7 @@ void db_read(const char * filename, struct Parameters const & p)
 
   uint64_t hdrhashsize {2 * sequences};
 
-  auto * * hdrhashtable =
-    static_cast<seqinfo_t **>(xmalloc(hdrhashsize * sizeof(seqinfo_t *)));
-  memset(hdrhashtable, 0, hdrhashsize * sizeof(seqinfo_t *));
-
-  uint64_t duplicatedidentifiers {0};
+  auto * * hdrhashtable = new seqinfo_t*[hdrhashsize] { };
 
   /* set up hash to check for unique sequences */
 
@@ -607,14 +645,12 @@ void db_read(const char * filename, struct Parameters const & p)
 
   if (p.opt_differences > 1)
     {
-      seqhashtable =
-        static_cast<seqinfo_t **>(xmalloc(seqhashsize * sizeof(seqinfo_t *)));
-      memset(seqhashtable, 0, seqhashsize * sizeof(seqinfo_t *));
+      seqhashtable = new seqinfo_t*[seqhashsize] { };
     }
 
   /* create indices */
 
-  seqindex = static_cast<seqinfo_t *>(xmalloc(sequences * sizeof(seqinfo_t)));
+  seqindex = new seqinfo_t[sequences];
   seqinfo_t * seqindex_p {seqindex};
 
   seqinfo_t * lastseq {nullptr};
@@ -642,11 +678,11 @@ void db_read(const char * filename, struct Parameters const & p)
       pl += nt_bytelength(seqlen);
 
       /* get amplicon abundance */
-      find_abundance(seqindex_p, line_number);
+      find_abundance(seqindex_p, line_number, p.opt_usearch_abundance, p.opt_append_abundance);
 
       if ((seqindex_p->abundance_start == 0) &&
           (seqindex_p->abundance_end == seqindex_p->headerlen)) {
-        fatal("Empty sequence identifier");
+        fatal(error_prefix, "Empty sequence identifier.");
       }
 
       /* check if the sequences are presorted by abundance and header */
@@ -726,11 +762,8 @@ void db_read(const char * filename, struct Parameters const & p)
 
       if (hdrfound != nullptr)
         {
-          duplicatedidentifiers++;
-          fprintf(stderr, "\nError: Duplicated sequence identifier: %.*s\n\n",
-                  id_len,
-                  seqindex_p->header + id_start);
-          exit(1);
+          const std::string full_header {seqindex_p->header + id_start};
+          fatal(error_prefix, "Duplicated sequence identifier: ", full_header.substr(0, id_len));
         }
 
       hdrhashtable[hdrhashindex] = seqindex_p;
@@ -774,33 +807,32 @@ void db_read(const char * filename, struct Parameters const & p)
 
   if (duplicates_found != 0U)
     {
-      fprintf(logfile,
-              "\n\n"
-              "Error: some fasta entries have identical sequences.\n"
-              "Swarm expects dereplicated fasta files.\n"
-              "Such files can be produced with swarm or vsearch:\n"
-              " swarm -d 0 -w derep.fasta -o /dev/null input.fasta\n"
-              "or\n"
-              " vsearch --derep_fulllength input.fasta --sizein --sizeout --output derep.fasta\n");
-      exit(1);
+      fatal(error_prefix,
+            "some fasta entries have identical sequences.\n"
+            "Swarm expects dereplicated fasta files.\n"
+            "Such files can be produced with swarm or vsearch:\n"
+            " swarm -d 0 -w derep.fasta -o /dev/null input.fasta\n"
+            "or\n"
+            " vsearch --derep_fulllength input.fasta --sizein --sizeout --output derep.fasta");
     }
 
   progress_done();
 
+  delete [] line;
+  line = nullptr;
+  linecap = 0;
+
   if (missingabundance != 0)
     {
-      fprintf(stderr,
-              "\nError: Abundance annotations not found for %d sequences, starting on line %" PRIu64 ".\n"
-              ">%s\n"
-              "Fasta headers must end with abundance annotations (_INT or ;size=INT).\n"
-              "The -z option must be used if the abundance annotation is in the latter format.\n"
-              "Abundance annotations can be produced by dereplicating the sequences.\n"
-              "The header is defined as the string comprised between the \">\" symbol\n"
-              "and the first space or the end of the line, whichever comes first.\n\n",
-              missingabundance,
-              missingabundance_lineno,
-              missingabundance_header);
-      exit(1);
+      fatal(error_prefix, "Abundance annotations not found for ",
+            missingabundance, " sequences, starting on line ",
+            missingabundance_lineno, ".\n>",
+            missingabundance_header, "\n",
+            "Fasta headers must end with abundance annotations (_INT or ;size=INT).\n"
+            "The -z option must be used if the abundance annotation is in the latter format.\n"
+            "Abundance annotations can be produced by dereplicating the sequences.\n"
+            "The header is defined as the string comprised between the \">\" symbol\n"
+            "and the first space or the end of the line, whichever comes first.");
     }
 
   if (!presorted)
@@ -810,19 +842,22 @@ void db_read(const char * filename, struct Parameters const & p)
       progress_done();
     }
 
-  xfree(hdrhashtable);
+  delete [] hdrhashtable;
+  hdrhashtable = nullptr;
 
-  if (seqhashtable != nullptr)
-    {
-      xfree(seqhashtable);
-      seqhashtable = nullptr;
-    }
+  delete [] seqhashtable;
+  seqhashtable = nullptr;
+
+  // user report
+  fprintf(logfile, "Database info:     %" PRIu64 " nt", db_getnucleotidecount());
+  fprintf(logfile, " in %u sequences,", db_getsequencecount());
+  fprintf(logfile, " longest %u nt\n", db_getlongestsequence());
 }
 
+
 void db_qgrams_init()
 {
-  qgrams = static_cast<qgramvector_t *>
-    (xmalloc(sequences * sizeof(qgramvector_t)));
+  qgrams = new qgramvector_t[sequences];
 
   seqinfo_t * seqindex_p {seqindex};
   progress_init("Find qgram vects: ", sequences);
@@ -838,36 +873,26 @@ void db_qgrams_init()
   progress_done();
 }
 
+
 void db_qgrams_done()
 {
-  xfree(qgrams);
+  delete [] qgrams;
+  qgrams = nullptr;
 }
 
-auto db_getsequencecount() -> unsigned int
-{
-  return sequences;
-}
-
-auto db_getnucleotidecount() -> uint64_t
-{
-  return nucleotides;
-}
-
-auto db_getlongestsequence() -> unsigned int
-{
-  return longest;
-}
 
 auto db_gethash(uint64_t seqno) -> uint64_t
 {
   return seqindex[seqno].seqhash;
 }
 
+
 auto db_getsequence(uint64_t seqno) -> char *
 {
   return seqindex[seqno].seq;
 }
 
+
 void db_getsequenceandlength(uint64_t seqno,
                              char ** address,
                              unsigned int * length)
@@ -876,98 +901,47 @@ void db_getsequenceandlength(uint64_t seqno,
   *length = seqindex[seqno].seqlen;
 }
 
+
 auto db_getsequencelen(uint64_t seqno) -> unsigned int
 {
   return seqindex[seqno].seqlen;
 }
 
+
 auto db_getheader(uint64_t seqno) -> char *
 {
   return seqindex[seqno].header;
 }
 
+
 auto db_getabundance(uint64_t seqno) -> uint64_t
 {
   return seqindex[seqno].abundance;
 }
 
+
 void db_free()
 {
   zobrist_exit();
 
-  if (datap != nullptr) {
-    xfree(datap);
-  }
-  if (seqindex != nullptr) {
-    xfree(seqindex);
-  }
+  delete [] datap;
+  datap = nullptr;
+  delete [] seqindex;
+  seqindex = nullptr;
 }
 
-void db_fprintseq(FILE * fp, unsigned int a, unsigned int width)
+
+auto db_fprintseq(std::FILE * fastaout_fp, const unsigned int seqno) -> void
 {
-  constexpr int default_length {1025};
-  char * seq = db_getsequence(a);
-  unsigned int len = db_getsequencelen(a);
-  char buffer[default_length];
-  char * buf {nullptr};
-
-  // buf = len < default_length ? buffer : static_cast<char*>(xmalloc(len+1));
-  if (len < default_length) {
-    buf = buffer;
-  }
-  else {
-    buf = static_cast<char*>(xmalloc(len+1));
-  }
+  const unsigned int len {db_getsequencelen(seqno)};
+  char * const seqptr {db_getsequence(seqno)};
+  static std::vector<char> buffer(db_getlongestsequence() + 1, '\0');
 
+  // decode to nucleotides (A, C, G and T)
   for(auto i = 0U; i < len; i++) {
-    buf[i] = sym_nt[1 + nt_extract(seq, i)];
+    buffer[i] = sym_nt[1 + nt_extract(seqptr, i)];
   }
-  buf[len] = 0;
+  buffer[len] = '\0';
 
-  if (width < 1) {
-    fprintf(fp, "%.*s\n", len, buf);
-  }
-  else {
-    auto rest = len;
-    for(auto i = 0U; i < len; i += width) {
-      fprintf(fp, "%.*s\n", std::min(rest, width), buf+i);
-      rest -= width;
-    }
-  }
-
-  if (len >= default_length) {
-    xfree(buf);
-  }
+  fprintf(fastaout_fp, "%.*s\n", len, buffer.data());
 }
-
-
-#if 0
-
-/* Unused functions */
-
-unsigned int db_getheaderlen(uint64_t seqno)
-{
-  return seqindex[seqno].headerlen;
-}
-
-unsigned int db_getlongestheader()
-{
-  return longestheader;
-}
-
-seqinfo_t * db_getseqinfo(uint64_t seqno)
-{
-  return seqindex+seqno;
-}
-
-void db_putseq(int64_t seqno)
-{
-  char * seq {nullptr};
-  int64_t len {0};
-  db_getsequenceandlength(seqno, & seq, & len);
-  for(auto i = 0LL; i < len; i++) {
-    putchar(sym_nt[1+nt_extract(seq, i)]);
-  }
-}
-
-#endif
