@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,48 +21,50 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
-#include "swarm.h"
+#include <cstdint>
+#include <cstring>  // memset
+#include "hashtable.h"
+#include "utils/hashtable_size.h"
+
 
 uint64_t hash_mask {0};
 unsigned char * hash_occupied {nullptr};
 uint64_t * hash_values {nullptr};
 unsigned int * hash_data {nullptr};
-uint64_t hash_tablesize {0};
 
-void hash_zap()
+
+void hash_zap(const uint64_t hashtablesize)
 {
   constexpr int padding {63};  // make sure our final value is >= 64 / 8
   constexpr int convert_to_bytes {8};
-  memset(hash_occupied, 0, (hash_tablesize + padding) / convert_to_bytes);
+  memset(hash_occupied, 0, (hashtablesize + padding) / convert_to_bytes);
 }
 
-void hash_alloc(uint64_t amplicons)
+
+auto hash_alloc(const uint64_t amplicons) -> uint64_t
 {
-  constexpr unsigned int hashfillpct {70};
-  constexpr unsigned int one_hundred_pct {100};
   constexpr int padding {63};  // make sure our final value is >= 64 / 8
   constexpr int convert_to_bytes {8};
-  hash_tablesize = 1;
-  // amplicons > 70% hash table size (avoid division to keep working with ints)
-  while (one_hundred_pct * amplicons > hashfillpct * hash_tablesize) {
-    hash_tablesize <<= 1;
-  }
-  hash_mask = hash_tablesize - 1;
 
-  hash_occupied =
-    static_cast<unsigned char *>(xmalloc((hash_tablesize + padding) / convert_to_bytes));
-  hash_zap();
+  const uint64_t hashtablesize {compute_hashtable_size(amplicons)};
+  hash_mask = hashtablesize - 1;
 
-  hash_values =
-    static_cast<uint64_t *>(xmalloc(hash_tablesize * sizeof(uint64_t)));
+  hash_occupied = new unsigned char[(hashtablesize + padding) / convert_to_bytes] { };
 
-  hash_data = static_cast<unsigned int *>
-    (xmalloc(hash_tablesize * sizeof(unsigned int)));
+  hash_values = new uint64_t[hashtablesize] { };
+
+  hash_data = new unsigned int[hashtablesize] { };
+
+  return hashtablesize;
 }
 
+
 void hash_free()
 {
-  xfree(hash_occupied);
-  xfree(hash_values);
-  xfree(hash_data);
+  delete [] hash_occupied;
+  hash_occupied = nullptr;
+  delete [] hash_values;
+  hash_values = nullptr;
+  delete [] hash_data;
+  hash_data = nullptr;
 }
