@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,16 +21,13 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
+#include <cstdint>
+
 extern uint64_t hash_mask;
 extern unsigned char * hash_occupied;
 extern uint64_t * hash_values;
 extern unsigned int * hash_data;
-extern uint64_t hash_tablesize;
 
-inline auto hash_get_tablesize() -> uint64_t
-{
-  return hash_tablesize;
-}
 
 inline auto hash_getindex(uint64_t hash) -> uint64_t
 {
@@ -79,8 +76,8 @@ inline void hash_set_data(uint64_t j, unsigned int x)
   hash_data[j] = x;
 }
 
-void hash_zap();
+void hash_zap(uint64_t hashtablesize);
 
-void hash_alloc(uint64_t amplicons);
+auto hash_alloc(uint64_t amplicons) -> uint64_t;
 
 void hash_free();
