@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,6 +22,16 @@
 */
 
 #include "swarm.h"
+#include "threads.h"
+#include "util.h"
+
+#ifdef __x86_64__
+
+#include "popcnt.h"
+
+#endif
+
+
 
 static ThreadRunner * qgram_threads = nullptr;
 
@@ -33,24 +43,6 @@ static struct thread_info_s
   uint64_t * difflist;
 } * ti;
 
-#if 0
-
-/* never used */
-
-void printqgrams(unsigned char * qgramvector)
-{
-  /* print qgramvector */
-  fprintf(logfile, "qgram vector:\n");
-  for(auto i = 0; i < qgramvectorbytes; i++)
-  {
-    fprintf(logfile, "%02x", qgramvector[i]);
-    if ((i % 32) == 31) {
-      fprintf(logfile, "\n");
-    }
-  }
-}
-
-#endif
 
 void findqgrams(unsigned char * seq, uint64_t seqlen,
                 unsigned char * qgramvector)
@@ -189,10 +181,10 @@ auto compareqgramvectors_128(unsigned char * a, unsigned char * b) -> uint64_t
 
 auto compareqgramvectors(unsigned char * a, unsigned char * b) -> uint64_t
 {
-  if (popcnt_present)
+  if (popcnt_present != 0) {
     return compareqgramvectors_popcnt(a, b);
-  else                                             \
-    return compareqgramvectors_128(a, b);
+  }
+  return compareqgramvectors_128(a, b);
 }
 
 #else
@@ -201,6 +193,11 @@ auto compareqgramvectors(unsigned char * a, unsigned char * b) -> uint64_t
 
 #endif
 
+inline auto db_getqgramvector(uint64_t seqno) -> unsigned char *
+{
+  return reinterpret_cast<unsigned char*>(qgrams + seqno);
+}
+
 inline auto qgram_diff(uint64_t a, uint64_t b) -> uint64_t
 {
   uint64_t diffqgrams = compareqgramvectors(db_getqgramvector(a),
@@ -226,9 +223,7 @@ void qgram_worker(int64_t t)
 void qgram_diff_init()
 {
   /* allocate memory for thread info */
-  ti = static_cast<struct thread_info_s *>
-    (xmalloc(static_cast<uint64_t>(opt_threads) *
-             sizeof(struct thread_info_s)));
+  ti = new struct thread_info_s[static_cast<uint64_t>(opt_threads)];
 
   qgram_threads
     = new ThreadRunner(static_cast<int> (opt_threads), qgram_worker);
@@ -237,8 +232,9 @@ void qgram_diff_init()
 void qgram_diff_done()
 {
   delete qgram_threads;
-
-  xfree(ti);
+  qgram_threads = nullptr;
+  delete [] ti;
+  ti = nullptr;
 }
 
 void qgram_diff_fast(uint64_t seed,
