@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,6 +22,11 @@
 */
 
 #include "swarm.h"
+#include "db.h"
+#include "search8.h"
+#include "search16.h"
+#include "threads.h"
+
 
 static pthread_mutex_t scan_mutex;
 
@@ -57,13 +62,6 @@ static uint64_t dirbufferbytes;
 queryinfo_t query;
 uint64_t longestdbsequence;
 
-void search_alloc(struct search_data * sdp);
-void search_free(struct search_data * sdp);
-void search_init(struct search_data * sdp);
-void search_chunk(struct search_data * sdp, int64_t bits);
-auto search_getwork(uint64_t * countref, uint64_t * firstref) -> bool;
-void search_worker_core(int64_t t);
-auto search_worker(void * vp) -> void *;
 
 void search_alloc(struct search_data * sdp)
 {
@@ -73,31 +71,28 @@ void search_alloc(struct search_data * sdp)
 
   dirbufferbytes =
     bytes_per_uint64 * longestdbsequence * ((longestdbsequence + 3) / 4) * 4;
-  sdp->qtable = static_cast<BYTE**>
-    (xmalloc(longestdbsequence * sizeof(BYTE*)));
-  sdp->qtable_w = static_cast<WORD**>
-    (xmalloc(longestdbsequence * sizeof(WORD*)));
-  sdp->dprofile = static_cast<BYTE*>
-    (xmalloc(2 * one_kilobyte));  // 4 * 16 * 32
-  sdp->dprofile_w = static_cast<WORD*>
-    (xmalloc(2 * one_kilobyte));  // 4 * 2 * 8 * 32
-  sdp->hearray = static_cast<BYTE*>
-    (xmalloc(longestdbsequence * nt_per_uint64));
-  sdp->dir_array = static_cast<uint64_t *>
-    (xmalloc(dirbufferbytes));
-
-  memset(sdp->hearray, 0, longestdbsequence * nt_per_uint64);
-  memset(sdp->dir_array, 0, dirbufferbytes);
+  sdp->qtable = new BYTE*[longestdbsequence];
+  sdp->qtable_w = new WORD*[longestdbsequence];
+  sdp->dprofile = new BYTE[2 * one_kilobyte];  // 4 * 16 * 32
+  sdp->dprofile_w = new WORD[2 * one_kilobyte];  // 4 * 2 * 8 * 32
+  sdp->hearray = new BYTE[longestdbsequence * nt_per_uint64] { };
+  sdp->dir_array = new uint64_t[dirbufferbytes] { };
 }
 
 void search_free(struct search_data * sdp)
 {
-  xfree(sdp->qtable);
-  xfree(sdp->qtable_w);
-  xfree(sdp->dprofile);
-  xfree(sdp->dprofile_w);
-  xfree(sdp->hearray);
-  xfree(sdp->dir_array);
+  delete [] sdp->qtable;
+  sdp->qtable = nullptr;
+  delete [] sdp->qtable_w;
+  sdp->qtable_w = nullptr;
+  delete [] sdp->dprofile;
+  sdp->dprofile = nullptr;
+  delete [] sdp->dprofile_w;
+  sdp->dprofile_w = nullptr;
+  delete [] sdp->hearray;
+  sdp->hearray = nullptr;
+  delete [] sdp->dir_array;
+  sdp->dir_array = nullptr;
 }
 
 void search_init(struct search_data * sdp)
@@ -107,54 +102,21 @@ void search_init(struct search_data * sdp)
 
   for(auto i = 0U; i < query.len; i++)
   {
-    int nt_value {nt_extract(query.seq, i) + 1};   // 1,  2,   3, or   4
-    int byte_offset {byte_multiplier * nt_value};  // 1, 64, 128, or 192
-    int word_offset {word_multiplier * nt_value};  // 1, 32,  64, or 128
+    const int nt_value {nt_extract(query.seq, i) + 1};   // 1,  2,   3, or   4
+    const int byte_offset {byte_multiplier * nt_value};  // 1, 64, 128, or 192
+    const int word_offset {word_multiplier * nt_value};  // 1, 32,  64, or 128
+
     sdp->qtable[i]   = sdp->dprofile   + byte_offset;
     sdp->qtable_w[i] = sdp->dprofile_w + word_offset;
   }
 }
 
-void search_chunk(struct search_data * sdp, int64_t bits)
+void search_chunk(struct search_data * sdp, const int64_t bits)
 {
   constexpr unsigned int bit_mode_16 {16};
-  if (sdp->target_count == 0) {
-    return;
-  }
 
-#if 0
-
-  for(auto i = 0ULL; i < sdp->target_count; i++)
-    {
-    char * dseq;
-    unsigned int dlen;
-    char * nwalignment;
-
-    uint64_t seqno = master_targets[sdp->target_index + i];
-    db_getsequenceandlength(seqno, & dseq, & dlen);
-
-    nw(dseq, dlen,
-       query.seq, query.len,
-       score_matrix_63,
-       penalty_gapopen, penalty_gapextend,
-       (int64_t *)(master_scores) + sdp->target_index + i,
-       (int64_t *)(master_diffs) + sdp->target_index + i,
-       (int64_t *)(master_alignlengths) + sdp->target_index + i,
-       & nwalignment,
-       (unsigned char *) sdp->dir_array,
-       (int64_t *) sdp->hearray,
-       query.qno, seqno);
-
-#if 0
-    printf("\nAlignment: %s\n", nwalignment);
-#endif
-
-    xfree(nwalignment);
-  }
-
-  return;
-
-#endif
+  assert(sdp->target_count != 0);
+  assert((bits == bit_mode_16) || (bits == bit_mode_16 / 2));
 
  if (bits == bit_mode_16)
    {
@@ -171,7 +133,8 @@ void search_chunk(struct search_data * sdp, int64_t bits)
              master_alignlengths + sdp->target_index,
              static_cast<uint64_t>(query.len),
              dirbufferbytes / sizeof(uint64_t),
-             sdp->dir_array);
+             sdp->dir_array,
+             longestdbsequence);
    }
  else {
     search8(sdp->qtable,
@@ -187,7 +150,8 @@ void search_chunk(struct search_data * sdp, int64_t bits)
             master_alignlengths + sdp->target_index,
             static_cast<uint64_t>(query.len),
             dirbufferbytes / sizeof(uint64_t),
-            sdp->dir_array);
+            sdp->dir_array,
+            longestdbsequence);
  }
 }
 
@@ -202,7 +166,7 @@ auto search_getwork(uint64_t * countref, uint64_t * firstref) -> bool
 
   if (master_next < master_length)
     {
-      uint64_t chunksize =
+      const uint64_t chunksize =
         ((master_length - master_next + remainingchunks - 1) / remainingchunks);
 
       * countref = chunksize;
@@ -218,45 +182,62 @@ auto search_getwork(uint64_t * countref, uint64_t * firstref) -> bool
   return status;
 }
 
-#if 0
 
-/* never used */
-
-void master_dump()
+void search_worker_core(const int64_t t)
 {
-  printf("master_dump\n");
-  printf("   i    t    s    d\n");
-  for(auto i = 0ULL; i < 1403; i++)
-    {
-      printf("%4" PRIu64 " %4" PRIu64 " %4" PRIu64 " %4" PRIu64 "\n",
-             i, master_targets[i], master_scores[i], master_diffs[i]);
-    }
-}
-
-#endif
-
-void search_worker_core(int64_t t)
-{
-  search_init(sd+t);
+  search_init(sd + t);
   while(search_getwork(& sd[t].target_count, & sd[t].target_index)) {
-    search_chunk(sd+t, master_bits);
+    search_chunk(sd + t, master_bits);
   }
 }
 
+
+auto adjust_thread_number(const int n_bits,
+                          const uint64_t remaining_sequences,
+                          uint64_t n_threads) -> uint64_t {
+  constexpr unsigned int channels_8 {8};
+  constexpr unsigned int channels_16 {16};
+  constexpr unsigned int bit_mode_16 {16};  // refactoring: should be an enum class
+  const auto channels { (n_bits == bit_mode_16) ? channels_8 : channels_16 };
+
+  assert(remaining_sequences != 0);
+  assert(n_threads != 0);
+  assert((n_threads - 1) <= (ULLONG_MAX / channels_8));
+  assert((n_bits == bit_mode_16) || (n_bits == bit_mode_16 / 2));
+
+  while (remaining_sequences <= (n_threads - 1) * channels) {
+    --n_threads;
+  }
+
+  return n_threads;
+}
+
+// arguments: bits, master_length, thr
+// static_assert(adjust_thread_number( 8, 32, 10) == 2);
+// static_assert(adjust_thread_number( 8, 32,  3) == 2);
+// static_assert(adjust_thread_number( 8, 31,  2) == 2);
+// static_assert(adjust_thread_number( 8, 17,  2) == 2);
+// static_assert(adjust_thread_number( 8, 16,  2) == 1);
+// static_assert(adjust_thread_number( 8,  1,  2) == 1);
+// static_assert(adjust_thread_number( 8, 32,  1) == 1);
+// static_assert(adjust_thread_number(16, 17, 10) == 3);
+// static_assert(adjust_thread_number(16, 17,  3) == 3);
+// static_assert(adjust_thread_number(16, 16,  3) == 2);
+// static_assert(adjust_thread_number(16, 15,  2) == 2);
+// static_assert(adjust_thread_number(16,  1,  3) == 1);
+// static_assert(adjust_thread_number(16, 17,  1) == 1);
+
+
 void search_do(uint64_t query_no,
                uint64_t listlength,
                uint64_t * targets,
                uint64_t * scores,
                uint64_t * diffs,
                uint64_t * alignlengths,
-               int bits)
+               const int bits)
 {
-  // constexpr unsigned int bit_mode_16 {16};
-  constexpr unsigned int channels_8 {8};
-  constexpr unsigned int bit_mode_8 {8};
-  constexpr unsigned int channels_16 {16};
-  query.qno = query_no;
   unsigned int query_len = 0;
+  query.qno = query_no;
   db_getsequenceandlength(query_no, &query.seq, &query_len);
   query.len = query_len;
 
@@ -268,20 +249,11 @@ void search_do(uint64_t query_no,
   master_alignlengths = alignlengths;
   master_bits = bits;
 
-  auto thr = static_cast<uint64_t>(opt_threads);
-
-  if (bits == bit_mode_8)
-    {
-      if (master_length <= (channels_16 - 1) * thr) {
-        thr = (master_length + channels_16 - 1) / channels_16;
-      }
-    }
-  else
-    {
-      if (master_length <= (channels_8 - 1) * thr) {
-        thr = (master_length + channels_8 - 1) / channels_8;
-      }
-    }
+  const auto thr {
+    adjust_thread_number(bits,
+                         master_length,
+                         static_cast<uint64_t>(opt_threads))
+  };
 
   remainingchunks = thr;
 
@@ -297,11 +269,10 @@ void search_begin()
 {
   longestdbsequence = db_getlongestsequence();
 
-  sd = static_cast<struct search_data *>
-    (xmalloc(sizeof(search_data) * static_cast<uint64_t>(opt_threads)));
+  sd = new struct search_data[static_cast<uint64_t>(opt_threads)];
 
   for(auto t = 0LL; t < opt_threads; t++) {
-    search_alloc(sd+t);
+    search_alloc(sd + t);
   }
 
   pthread_mutex_init(& scan_mutex, nullptr);
@@ -309,7 +280,7 @@ void search_begin()
   /* start threads */
 
   search_threads
-    = new ThreadRunner(static_cast<int> (opt_threads), search_worker_core);
+    = new ThreadRunner(static_cast<int>(opt_threads), search_worker_core);
 }
 
 void search_end()
@@ -317,11 +288,13 @@ void search_end()
   /* finish and clean up worker threads */
 
   delete search_threads;
+  search_threads = nullptr;
 
   pthread_mutex_destroy(& scan_mutex);
 
   for(auto t = 0LL; t < opt_threads; t++) {
-    search_free(sd+t);
+    search_free(sd + t);
   }
-  xfree(sd);
+  delete [] sd;
+  sd = nullptr;
 }
