@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,9 +22,20 @@
 */
 
 #include "swarm.h"
+#include "db.h"
+
+#ifdef __x86_64__
+
+#include "ssse3.h"
+#include "sse41.h"
+
+#endif
 
 constexpr unsigned int channels {8};
 constexpr unsigned int cdepth {4};
+using BYTE = unsigned char;
+using WORD = unsigned short;
+
 
 /* uses 8 unsigned 16-bit values */
 
@@ -135,69 +146,6 @@ const vector unsigned char perm_bits =
 
 #endif
 
-void align_cells_regular_16(VECTORTYPE * Sm,
-                            VECTORTYPE * hep,
-                            VECTORTYPE ** qp,
-                            VECTORTYPE * Qm,
-                            VECTORTYPE * Rm,
-                            uint64_t ql,
-                            VECTORTYPE * F0,
-                            uint64_t * dir_long,
-                            VECTORTYPE * H0);
-
-void align_cells_masked_16(VECTORTYPE * Sm,
-                           VECTORTYPE * hep,
-                           VECTORTYPE ** qp,
-                           VECTORTYPE * Qm,
-                           VECTORTYPE * Rm,
-                           uint64_t ql,
-                           VECTORTYPE * F0,
-                           uint64_t * dir_long,
-                           VECTORTYPE * H0,
-                           VECTORTYPE * Mm,
-                           VECTORTYPE * MQ,
-                           VECTORTYPE * MR,
-                           VECTORTYPE * MQ0);
-
-#ifdef __x86_64__
-
-/* functions defined in sse41.cc */
-
-void align_cells_regular_16_sse41(VECTORTYPE * Sm,
-                                  VECTORTYPE * hep,
-                                  VECTORTYPE ** qp,
-                                  VECTORTYPE * Qm,
-                                  VECTORTYPE * Rm,
-                                  uint64_t ql,
-                                  VECTORTYPE * F0,
-                                  uint64_t * dir_long,
-                                  VECTORTYPE * H0);
-
-void align_cells_masked_16_sse41(VECTORTYPE * Sm,
-                                 VECTORTYPE * hep,
-                                 VECTORTYPE ** qp,
-                                 VECTORTYPE * Qm,
-                                 VECTORTYPE * Rm,
-                                 uint64_t ql,
-                                 VECTORTYPE * F0,
-                                 uint64_t * dir_long,
-                                 VECTORTYPE * H0,
-                                 VECTORTYPE * Mm,
-                                 VECTORTYPE * MQ,
-                                 VECTORTYPE * MR,
-                                 VECTORTYPE * MQ0);
-
-#endif
-
-auto backtrack_16(char * qseq,
-                  char * dseq,
-                  uint64_t qlen,
-                  uint64_t dlen,
-                  uint64_t * dirbuffer,
-                  uint64_t offset,
-                  uint64_t dirbuffersize,
-                  uint64_t channel,
-                  uint64_t * alignmentlengthp) -> uint64_t;
 
 inline void dprofile_fill16(WORD * dprofile_word,
                             WORD * score_matrix_word,
@@ -499,14 +447,15 @@ void align_cells_masked_16(VECTORTYPE * Sm,
 }
 
 auto backtrack_16(char * qseq,
-                      char * dseq,
-                      uint64_t qlen,
-                      uint64_t dlen,
-                      uint64_t * dirbuffer,
-                      uint64_t offset,
-                      uint64_t dirbuffersize,
-                      uint64_t channel,
-                      uint64_t * alignmentlengthp) -> uint64_t
+                  char * dseq,
+                  uint64_t qlen,
+                  uint64_t dlen,
+                  uint64_t * dirbuffer,
+                  uint64_t offset,
+                  uint64_t dirbuffersize,
+                  uint64_t channel,
+                  uint64_t * alignmentlengthp,
+                  const uint64_t longestdbsequence) -> uint64_t
 {
   constexpr unsigned int offset0 {0};
   constexpr unsigned int offset1 {offset0 + 16};
@@ -517,61 +466,6 @@ auto backtrack_16(char * qseq,
   uint64_t maskextup   = 3ULL << (2 * channel + offset2);
   uint64_t maskextleft = 3ULL << (2 * channel + offset3);
 
-#if 0
-
-  printf("Dumping backtracking array\n");
-
-  for(auto i = 0ULL; i < qlen; i++)
-    {
-      for(auto j = 0ULL; j < dlen; j++)
-        {
-          uint64_t d = dirbuffer[(offset + longestdbsequence * 4 * (j / 4)
-                                  + 4 * i + (j & 3)) % dirbuffersize];
-          if (d & maskleft)
-            {
-              printf("<");
-            }
-          else if (!(d & maskup))
-            {
-              printf("^");
-            }
-          else
-            {
-              printf("\\");
-            }
-        }
-      printf("\n");
-    }
-
-  printf("Dumping gap extension array\n");
-
-  for(auto i = 0ULL; i < qlen; i++)
-    {
-      for(auto j = 0ULL; j < dlen; j++)
-        {
-          uint64_t d = dirbuffer[(offset + longestdbsequence * 4 *(j / 4)
-                                  + 4 * i + (j & 3)) % dirbuffersize];
-          if (!(d & maskextup))
-            {
-              if (!(d & maskextleft))
-                printf("+");
-              else
-                printf("^");
-            }
-          else if (!(d & maskextleft))
-            {
-              printf("<");
-            }
-          else
-            {
-              printf("\\");
-            }
-        }
-      printf("\n");
-    }
-
-#endif
-
   int64_t i = static_cast<int64_t>(qlen) - 1;
   int64_t j = static_cast<int64_t>(dlen) - 1;
   uint64_t aligned {0};
@@ -666,7 +560,8 @@ void search16(WORD * * q_start,
               uint64_t * alignmentlengths,
               uint64_t qlen,
               uint64_t dirbuffersize,
-              uint64_t * dirbuffer)
+              uint64_t * dirbuffer,
+              const uint64_t longestdbsequence)
 {
   VECTORTYPE Q;
   VECTORTYPE R;
@@ -765,7 +660,7 @@ void search16(WORD * * q_start,
             }
 
 #ifdef __x86_64__
-          if (sse41_present)
+          if (sse41_present != 0)
             {
               align_cells_regular_16_sse41(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0);
             }
@@ -833,7 +728,8 @@ void search16(WORD * * q_start,
                                               dirbuffer,
                                               offset,
                                               dirbuffersize, c,
-                                              alignmentlengths + cand_id);
+                                              alignmentlengths + cand_id,
+                                              longestdbsequence);
                         }
                       else
                         {
@@ -915,7 +811,7 @@ void search16(WORD * * q_start,
           MQ0 = MQ;
 
 #ifdef __x86_64__
-          if (sse41_present)
+          if (sse41_present != 0)
             {
               align_cells_masked_16_sse41(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0, &M, &MQ, &MR, &MQ0);
             }
