@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2022 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -21,6 +21,8 @@
     PO Box 1080 Blindern, NO-0316 Oslo, Norway
 */
 
+#include <cstdint>
+
 extern uint64_t * zobrist_tab_base;
 extern uint64_t * zobrist_tab_byte_base;
 
@@ -28,11 +30,12 @@ void zobrist_init(unsigned int n);
 
 void zobrist_exit();
 
-auto zobrist_hash(unsigned char * s, unsigned int len) -> uint64_t;
-auto zobrist_hash_delete_first(unsigned char * s, unsigned int len) -> uint64_t;
-auto zobrist_hash_insert_first(unsigned char * s, unsigned int len) -> uint64_t;
+auto zobrist_hash(unsigned char * seq, unsigned int len) -> uint64_t;
+auto zobrist_hash_delete_first(unsigned char * seq, unsigned int len) -> uint64_t;
+auto zobrist_hash_insert_first(unsigned char * seq, unsigned int len) -> uint64_t;
 
-inline auto zobrist_value(unsigned int pos, unsigned char x) -> uint64_t
+inline auto zobrist_value(const unsigned int pos, const unsigned char offset) -> uint64_t
 {
-  return zobrist_tab_base[4 * pos + x];
+  // offset can be 0, 1 or 2 (remainder of uint64 % 3)
+  return zobrist_tab_base[4 * pos + offset];
 }
